@@ -1,26 +1,25 @@
-#![allow(unstable)]
+#![feature(core)]
 #![deny(missing_docs, warnings)]
 
 //! Lazily-Evaluated, Order-Independent Plugins for Extensible Types.
 
 extern crate typemap;
-extern crate phantom;
 use typemap::{TypeMap, Key};
 
-pub use phantom::Phantom;
-
 /// Implementers of this trait can act as plugins for other types, via `OtherType::get<P>()`.
 ///
 /// To create a plugin, implement this trait and provide an empty implementation
 /// of `Key` to associate the plugin with its return type, `Key::Value`.
 pub trait Plugin<E: ?Sized>: Key {
+    type Error;
+
     /// Create the plugin from an instance of the extended type.
     ///
     /// While `eval` is given a mutable reference to the extended
     /// type, it is important for implementers to remember that
     /// the result of `eval` is usually cached, so care should
     /// be taken when doing mutation on the extended type.
-    fn eval(&mut E, Phantom<Self>) -> Option<Self::Value>;
+    fn eval(&mut E) -> Result<Self::Value, Self::Error>;
 }
 
 /// Defines an interface that extensible types must implement.
@@ -41,21 +40,21 @@ pub trait Pluggable {
     /// Return a copy of the plugin's produced value.
     ///
     /// The plugin will be created if it doesn't exist already.
-    /// If plugin creation fails, `None` is returned.
+    /// If plugin creation fails, an error is returned.
     ///
     /// `P` is the plugin type.
-    fn get<P: Plugin<Self>>(&mut self) -> Option<P::Value>
-    where P::Value: Clone + 'static, Self: Extensible {
-        self.get_ref::<P>().cloned()
+    fn get<P: Plugin<Self>>(&mut self) -> Result<P::Value, P::Error>
+    where P::Value: Clone + 'static, Self: Extensible, P::Error: Clone {
+        self.get_ref::<P>().map(|v| v.clone())
     }
 
     /// Return a reference to the plugin's produced value.
     ///
     /// The plugin will be created if it doesn't exist already.
-    /// If plugin creation fails, `None` is returned.
+    /// If plugin creation fails an error is returned.
     ///
     /// `P` is the plugin type.
-    fn get_ref<P: Plugin<Self>>(&mut self) -> Option<&P::Value>
+    fn get_ref<P: Plugin<Self>>(&mut self) -> Result<&P::Value, P::Error>
     where P::Value: 'static, Self: Extensible {
         self.get_mut::<P>().map(|mutref| &*mutref)
     }
@@ -63,19 +62,19 @@ pub trait Pluggable {
     /// Return a mutable reference to the plugin's produced value.
     ///
     /// The plugin will be created if it doesn't exist already.
-    /// If plugin creation fails, `None` is returned.
+    /// If plugin creation fail an error is returned.
     ///
     /// `P` is the plugin type.
-    fn get_mut<P: Plugin<Self>>(&mut self) -> Option<&mut P::Value>
+    fn get_mut<P: Plugin<Self>>(&mut self) -> Result<&mut P::Value, P::Error>
     where P::Value: 'static, Self: Extensible {
         use typemap::Entry::{Occupied, Vacant};
         use std::intrinsics::unreachable;
 
         if self.extensions().contains::<P>() {
-            return self.extensions_mut().get_mut::<P>();
+            return Ok(self.extensions_mut().get_mut::<P>().unwrap());
         }
 
-        Plugin::eval(self, Phantom::<P>).map(move |data| {
+        <P as Plugin<Self>>::eval(self).map(move |data| {
             match self.extensions_mut().entry::<P>() {
                 Vacant(entry) => entry.insert(data),
                 Occupied(..) => unsafe { unreachable() }
@@ -84,15 +83,18 @@ pub trait Pluggable {
     }
 
     /// Create and evaluate a once-off instance of a plugin.
-    fn compute<P: Plugin<Self>>(&mut self) -> Option<P::Value> {
-        Plugin::eval(self, Phantom::<P>)
+    fn compute<P: Plugin<Self>>(&mut self) -> Result<P::Value, P::Error> {
+        <P as Plugin<Self>>::eval(self)
     }
 }
 
 #[cfg(test)]
 mod test {
+    extern crate void;
+
+    use test::void::{Void, VoidExtensions};
+
     use typemap::{TypeMap, Key};
-    use phantom::Phantom;
     use super::{Extensible, Plugin, Pluggable};
 
     struct Extended {
@@ -114,14 +116,16 @@ mod test {
 
     macro_rules! generate_simple_plugin (
         ($t:ty, $v:ident, $v2:expr) => {
-            #[derive(PartialEq, Show, Clone)]
+            #[derive(PartialEq, Debug, Clone)]
             struct $v(i32);
 
             impl Key for $t { type Value = $t; }
 
             impl Plugin<Extended> for $t {
-                fn eval(_: &mut Extended, _: Phantom<$t>) -> Option<$t> {
-                    Some($v($v2))
+                type Error = Void;
+
+                fn eval(_: &mut Extended) -> Result<$t, Void> {
+                    Ok($v($v2))
                 }
             }
         }
@@ -140,24 +144,24 @@ mod test {
 
     #[test] fn test_simple() {
         let mut extended = Extended::new();
-        assert_eq!(extended.get::<One>(),   Some(One(1)));
-        assert_eq!(extended.get::<Two>(),   Some(Two(2)));
-        assert_eq!(extended.get::<Three>(), Some(Three(3)));
+        assert_eq!(extended.get::<One>(),   Ok(One(1)));
+        assert_eq!(extended.get::<Two>(),   Ok(Two(2)));
+        assert_eq!(extended.get::<Three>(), Ok(Three(3)));
     }
 
     #[test] fn test_resize() {
         let mut extended = Extended::new();
-        extended.get::<One>();
-        extended.get::<Two>();
-        extended.get::<Three>();
-        extended.get::<Four>();
-        extended.get::<Five>();
-        extended.get::<Six>();
-        extended.get::<Seven>();
-        extended.get::<Eight>();
-        extended.get::<Nine>();
-        extended.get::<Ten>();
-        assert_eq!(extended.get_ref::<One>(), Some(&One(1)))
+        extended.get::<One>().void_unwrap();
+        extended.get::<Two>().void_unwrap();
+        extended.get::<Three>().void_unwrap();
+        extended.get::<Four>().void_unwrap();
+        extended.get::<Five>().void_unwrap();
+        extended.get::<Six>().void_unwrap();
+        extended.get::<Seven>().void_unwrap();
+        extended.get::<Eight>().void_unwrap();
+        extended.get::<Nine>().void_unwrap();
+        extended.get::<Ten>().void_unwrap();
+        assert_eq!(extended.get_ref::<One>(), Ok(&One(1)))
     }
 
     #[test] fn test_custom_return_type() {
@@ -171,11 +175,13 @@ mod test {
 
         // Define the plugin evaluation function.
         impl Plugin<Extended> for IntPlugin {
-            fn eval(_: &mut Extended, _: Phantom<IntPlugin>) -> Option<i32> {
-                Some(0i32)
+            type Error = Void;
+
+            fn eval(_: &mut Extended) -> Result<i32, Void> {
+                Ok(0i32)
             }
         }
-        assert_eq!(extended.get::<IntPlugin>().unwrap(), 0i32);
+        assert_eq!(extended.get::<IntPlugin>().void_unwrap(), 0i32);
     }
 }
 
